@@ -1,4 +1,6 @@
 import kubernetes
+from dagster_k8s.launcher import K8sRunLauncher
+
 from dagster import Field, StringSource, check, executor
 from dagster.core.definitions.executor_definition import multiple_process_executor_requirements
 from dagster.core.errors import DagsterUnmetExecutorRequirementsError
@@ -12,7 +14,6 @@ from dagster.core.executor.step_delegating.step_handler import StepHandler
 from dagster.core.executor.step_delegating.step_handler.base import StepHandlerContext
 from dagster.core.types.dagster_type import Optional
 from dagster.utils import frozentags, merge_dicts
-from dagster_k8s.launcher import K8sRunLauncher
 
 from .job import (
     DagsterK8sJobConfig,
@@ -55,6 +56,7 @@ def k8s_job_executor(init_context: InitExecutorContext) -> Executor:
             service_account_name: ...
             env_config_maps: ...
             env_secrets: ...
+            env_vars: ...
             job_image: ... # leave out if using userDeployments
 
     Configuration set on the Kubernetes Jobs and Pods created by the `K8sRunLauncher` will also be
@@ -88,6 +90,7 @@ def k8s_job_executor(init_context: InitExecutorContext) -> Executor:
         ),
         env_config_maps=run_launcher.env_config_maps + (exc_cfg.get("env_config_maps") or []),
         env_secrets=run_launcher.env_secrets + (exc_cfg.get("env_secrets") or []),
+        env_vars=run_launcher.env_vars + (exc_cfg.get("env_vars") or []),
         volume_mounts=run_launcher.volume_mounts + (exc_cfg.get("volume_mounts") or []),
         volumes=run_launcher.volumes + (exc_cfg.get("volumes") or []),
         labels=merge_dicts(run_launcher.labels, exc_cfg.get("labels", {})),
@@ -153,9 +156,9 @@ class K8sStepHandler(StepHandler):
         if step_handler_context.execute_step_args.known_state:
             retry_state = step_handler_context.execute_step_args.known_state.get_retry_state()
             if retry_state.get_attempt_count(step_key):
-                return "dagster-job-%s-%d" % (name_key, retry_state.get_attempt_count(step_key))
+                return "dagster-step-%s-%d" % (name_key, retry_state.get_attempt_count(step_key))
 
-        return "dagster-job-%s" % (name_key)
+        return "dagster-step-%s" % (name_key)
 
     def launch_step(self, step_handler_context: StepHandlerContext):
         events = []
