@@ -1,6 +1,7 @@
 import sys
 
 import kubernetes
+
 from dagster import Field, MetadataEntry, StringSource, check
 from dagster.cli.api import ExecuteRunArgs
 from dagster.core.events import EngineEventData
@@ -138,6 +139,10 @@ class K8sRunLauncher(RunLauncher, ConfigurableClass):
     def volumes(self):
         return self._volumes
 
+    @property
+    def env_vars(self):
+        return self._env_vars
+
     @property
     def labels(self):
         return self._labels
@@ -392,4 +397,6 @@ class K8sRunLauncher(RunLauncher, ConfigurableClass):
             )
         if job.status.failed:
             return CheckRunHealthResult(WorkerStatus.FAILED, "K8s job failed")
+        if job.status.succeeded:
+            return CheckRunHealthResult(WorkerStatus.SUCCESS)
         return CheckRunHealthResult(WorkerStatus.RUNNING)
